/*!
    @header PJS_Types
    @abstract Definitions of types used throughout the library
*/

#ifndef __PJS_TYPES_H__
#define __PJS_TYPES_H__

#ifdef __cplusplus
extern "C" {
#endif

typedef struct PJS_Context PJS_Context;
typedef struct PJS_Function PJS_Function;
typedef struct PJS_Property PJS_Property;
typedef struct PJS_Class PJS_Class;
typedef struct PJS_TrapHandler PJS_TrapHandler;
typedef struct PJS_Runtime PJS_Runtime;
typedef struct PJS_Script PJS_Script;
typedef struct PJS_PerlArray PJS_PerlArray;
typedef struct PJS_PerlHash PJS_PerlHash;
typedef struct PJS_PerlSub PJS_PerlSub;

#ifdef __cplusplus
}
#endif

#endif

