/* This file is autogenerated to suite your platform */

#ifndef __JAVASCRIPT_ENV_H__
#define __JAVASCRIPT_ENV_H__

#define XP_UNIX

#include <jsapi.h>
#include <jsdbgapi.h>
#include <jsinterp.h>
#include <jsfun.h>
#include <jsobj.h>
#include <jsprf.h>
#include <jsscope.h>

#endif
